import tkinter as tk
from tkinter import filedialog, messagebox, ttk
import pandas as pd
import numpy as np
import joblib
import subprocess

model = joblib.load("dietitian_model.pkl")

REQUIRED_COLUMNS = ["end_tidal_co2", "feed_vol", "feed_vol_adm", "fio2", "fio2_ratio", 
                    "insp_time", "oxygen_flow_rate", "peep", "pip", "resp_rate", "sip", 
                    "tidal_vol", "tidal_vol_actual", "tidal_vol_kg", "tidal_vol_spon", "bmi"]

class UploadPage(tk.Frame):
    def __init__(self, parent, controller):
        super().__init__(parent)
        self.controller = controller
        
        self.sidebar = tk.Frame(self, bg='#2c3e50', width=220, height=800)
        self.sidebar.pack(side=tk.LEFT, fill=tk.Y)
        
        self.home_btn = tk.Button(self.sidebar, text="Homepage", command=lambda: self.controller.show_frame("Homepage"), bg='#3498db', font=("Arial", 12))
        self.home_btn.pack(pady=10, padx=10, fill=tk.X)

        self.upload_btn = tk.Button(self.sidebar, text="Upload CSV", command=self.upload_csv, bg='#27ae60', font=("Arial", 12))
        self.upload_btn.pack(pady=10, padx=10, fill=tk.X)
        
        self.main_frame = tk.Frame(self)
        self.main_frame.pack(fill="both", expand=True)

        self.filter_frame = tk.Frame(self.main_frame)
        self.filter_frame.pack(side="top", fill="x", padx=10, pady=5)

        self.search_var = tk.StringVar()
        self.search_entry = tk.Entry(self.filter_frame, textvariable=self.search_var, width=20)
        self.search_entry.pack(side="left", padx=5)
        self.search_entry.bind("<KeyRelease>", self.search_patient)

        self.show_all_btn = tk.Button(self.filter_frame, text="Show All", command=self.show_all)
        self.show_all_btn.pack(side="left", padx=5)

        self.need_referral_btn = tk.Button(self.filter_frame, text="Need Referral", command=lambda: self.filter_by_status("Need Referral"))
        self.need_referral_btn.pack(side="left", padx=5)

        self.no_need_referral_btn = tk.Button(self.filter_frame, text="No Need Referral", command=lambda: self.filter_by_status("No Need Referral"))
        self.no_need_referral_btn.pack(side="left", padx=5)

        self.table_frame = tk.Frame(self.main_frame)
        self.table_frame.pack(side="left", fill="both", expand=True, padx=10, pady=10)

        self.tree = ttk.Treeview(self.table_frame, columns=("ID", "Referral"), show="headings")
        self.tree.heading("ID", text="Patient ID")
        self.tree.heading("Referral", text="Dietitian Referral")
        self.tree.bind("<Double-1>", self.view_patient_details)  
        self.tree.pack(fill="both", expand=True)

        self.details_frame = tk.Frame(self.main_frame, relief=tk.SUNKEN, borderwidth=2)
        self.details_visible = False

        tk.Button(self, text="Upload New CSV", command=self.upload_csv).pack(pady=5)

        self.patients = []

    def open_homepage(self):
        self.controller.show_frame("Homepage")  

    def close_patient_details(self):
        """Hide patient details panel."""
        if self.details_visible:
            self.details_frame.pack_forget()
            self.details_visible = False

    def view_patient_details(self, event):
        """Always show details when a patient is clicked."""
        selected_item = self.tree.selection()
        if not selected_item:
            return

        patient_id = int(float(self.tree.item(selected_item, "values")[0]))

        patient = next((p for p in self.patients if p["encounterId"] == patient_id), None)
        if patient:
            for widget in self.details_frame.winfo_children():
                widget.destroy()

            self.details_label = tk.Label(self.details_frame, text="Patient Details", font=("Arial", 14, "bold"))
            self.details_label.pack(anchor="n", pady=5)

            close_button = tk.Button(self.details_frame, text="✖", font=("Arial", 12, "bold"), 
                                     command=self.close_patient_details, relief=tk.FLAT, 
                                     bg="white", fg="gray", width=2, height=1)
            close_button.pack(anchor="ne", padx=3, pady=3)

            self.details_table = ttk.Treeview(self.details_frame, columns=("Attribute", "Value"), show="headings")
            self.details_table.heading("Attribute", text="Attribute")
            self.details_table.heading("Value", text="Value")
            self.details_table.pack(fill="both", expand=True, padx=5, pady=5)

            referral_status = "Need Referral" if patient['referral'] else "No Need Referral"
            self.details_table.insert("", "end", values=("Patient ID", patient['encounterId']))
            self.details_table.insert("", "end", values=("Referral Status", referral_status), tags=(referral_status,))
            self.details_table.tag_configure("Need Referral", background="#FFDDDD")
            self.details_table.tag_configure("No Need Referral", background="#DDEEFF")

            for key in REQUIRED_COLUMNS:
                self.details_table.insert("", "end", values=(key.replace('_', ' ').title(), patient[key]))

            self.details_frame.pack(side="right", fill="both", expand=True, padx=10, pady=10)
            self.details_visible = True

    def search_patient(self, event=None):
        """Search by Patient ID dynamically."""
        search_query = self.search_var.get().strip()
        self.tree.delete(*self.tree.get_children())
        for patient in self.patients:
            if search_query.isdigit() and str(patient["encounterId"]).startswith(search_query):
                referral_status = "Need Referral" if patient["referral"] else "No Need Referral"
                self.tree.insert("", "end", values=(patient["encounterId"], referral_status), tags=(referral_status,))

    def filter_by_status(self, status):
        """Filter patients by referral status."""
        self.tree.delete(*self.tree.get_children())
        for patient in self.patients:
            referral_status = "Need Referral" if patient["referral"] else "No Need Referral"
            if referral_status == status:
                self.tree.insert("", "end", values=(patient["encounterId"], referral_status), tags=(referral_status,))

    def show_all(self):
        """Show all patients."""
        self.tree.delete(*self.tree.get_children())
        for patient in self.patients:
            referral_status = "Need Referral" if patient["referral"] else "No Need Referral"
            self.tree.insert("", "end", values=(patient["encounterId"], referral_status), tags=(referral_status,))

    def upload_csv(self):
        """Upload CSV and update table."""
        file_path = filedialog.askopenfilename(filetypes=[("CSV Files", "*.csv")])
        if not file_path:
            return

        df = pd.read_csv(file_path)

        if "encounterId" in df.columns:
           df["encounterId"] = df["encounterId"].astype(int)

        trained_feature_names = list(model.feature_names_in_)
        missing_cols = [col for col in trained_feature_names if col not in df.columns]
        if missing_cols:
            messagebox.showerror("Error", f"Missing columns: {', '.join(missing_cols)}")
            return

        df = df[["encounterId"] + trained_feature_names]
        self.patients.clear()
        self.tree.delete(*self.tree.get_children())

        feature_values = df[trained_feature_names].to_numpy()
        referral_predictions = model.predict(feature_values)

        for i, row in df.iterrows():
            referral_prediction = int(referral_predictions[i])
            patient_entry = {"encounterId": int(row["encounterId"]), "referral": referral_prediction}
            for key in trained_feature_names:
                patient_entry[key] = row[key]

            self.patients.append(patient_entry)
            referral_status = "Need Referral" if referral_prediction else "No Need Referral"
            self.tree.insert("", "end", values=(int(row["encounterId"]), referral_status), tags=(referral_status,))

        self.tree.tag_configure("Need Referral", background="#FFDDDD")
        self.tree.tag_configure("No Need Referral", background="#DDEEFF")

        df["referral"] = referral_predictions  
        df.to_csv("data.csv", index=False) 

        messagebox.showinfo("Success", "CSV uploaded successfully!")